﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;

namespace TestProject
{
    /// <summary>
    /// 클래스 계층 트리 뷰
    /// </summary>
    public class ClassHierarchyTreeView : TreeView
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ClassHierarchyTreeView(rootType)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="rootType">루트 타입</param>
        public ClassHierarchyTreeView(Type rootType)
        {
            UIElement dummyElement = new UIElement();

            List<Assembly> assemblyList = new List<Assembly>();

            AssemblyName[] assemblyNameArray = Assembly.GetExecutingAssembly().GetReferencedAssemblies();

            foreach(AssemblyName assemblyName in assemblyNameArray)
            {
                assemblyList.Add(Assembly.Load(assemblyName));
            }

            SortedList<string, Type> classList = new SortedList<string, Type>();

            classList.Add(rootType.Name, rootType);

            foreach(Assembly assembly in assemblyList)
            {
                foreach(Type type in assembly.GetTypes())
                {
                    if(type.IsPublic && type.IsSubclassOf(rootType))
                    {
                        classList.Add(type.Name, type);
                    }
                }
            }

            TypeTreeViewItem rootTypeItem = new TypeTreeViewItem(rootType);

            Items.Add(rootTypeItem);

            AddDerivedClass(rootTypeItem, classList);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 파생 클래스 추가하기 - AddDerivedClass(parentItem, classList)

        /// <summary>
        /// 파생 클래스 추가하기
        /// </summary>
        /// <param name="parentItem">부모 항목</param>
        /// <param name="classList">클래스 리스트</param>
        private void AddDerivedClass(TypeTreeViewItem parentItem, SortedList<string, Type> classList)
        {
            foreach(KeyValuePair<string, Type> keyValuePair in classList)
            {
                if(keyValuePair.Value.BaseType == parentItem.Type)
                {
                    TypeTreeViewItem childItem = new TypeTreeViewItem(keyValuePair.Value);

                    parentItem.Items.Add(childItem);

                    AddDerivedClass(childItem, classList);
                }
            }
        }

        #endregion
    }
}