﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;

namespace TestProject
{
    /// <summary>
    /// 의존 속성 리스트 뷰
    /// </summary>
    public class DependencyPropertyListView : ListView
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Dependency Property
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 타입 속성 - TypeProperty

        /// <summary>
        /// 타입 속성
        /// </summary>
        public static DependencyProperty TypeProperty = DependencyProperty.Register
        (
            "Type",
            typeof(Type),
            typeof(DependencyPropertyListView),
            new PropertyMetadata(null, new PropertyChangedCallback(TypePropertyChangedCallback))
        );

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 타입 - Type

        /// <summary>
        /// 타입
        /// </summary>
        public Type Type
        {
            set
            {
                SetValue(TypeProperty, value);
            }
            get
            {
                return (Type)GetValue(TypeProperty);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - DependencyPropertyListView()

        /// <summary>
        /// 생성자
        /// </summary>
        public DependencyPropertyListView()
        {
            GridView gridView = new GridView();

            View = gridView;

            #region Name 그리드 뷰 컬럼

            GridViewColumn nameGridViewColumn = new GridViewColumn();

            nameGridViewColumn.Header               = "Name";
            nameGridViewColumn.Width                = double.NaN;
            nameGridViewColumn.DisplayMemberBinding = new Binding("Name");

            gridView.Columns.Add(nameGridViewColumn);

            #endregion
            #region OwnerType 그리드 뷰 컬럼

            GridViewColumn ownerTypeGridViewColumn = new GridViewColumn();

            ownerTypeGridViewColumn.Header = "Owner";
            ownerTypeGridViewColumn.Width  = double.NaN;

            gridView.Columns.Add(ownerTypeGridViewColumn);

            DataTemplate ownerTypeDataTemplate = new DataTemplate();

            ownerTypeGridViewColumn.CellTemplate = ownerTypeDataTemplate;

            FrameworkElementFactory ownerTypeTextBlockFrameworkElementFactory = new FrameworkElementFactory(typeof(TextBlock));

            ownerTypeDataTemplate.VisualTree = ownerTypeTextBlockFrameworkElementFactory;

            Binding ownerTypeBinding = new Binding("OwnerType");

            ownerTypeBinding.Converter = new TypeToTypeNameValueConverter();

            ownerTypeTextBlockFrameworkElementFactory.SetBinding(TextBlock.TextProperty, ownerTypeBinding);

            #endregion
            #region PropertyType 그리드 뷰 컬럼

            GridViewColumn propertyTypeGridViewColumn = new GridViewColumn();

            propertyTypeGridViewColumn.Header = "Type";
            propertyTypeGridViewColumn.Width  = double.NaN;

            gridView.Columns.Add(propertyTypeGridViewColumn);

            DataTemplate propertyTypeDataTemplate = new DataTemplate();

            propertyTypeGridViewColumn.CellTemplate = propertyTypeDataTemplate;

            FrameworkElementFactory propertyTypeTextBlockFrameworkElementFactory = new FrameworkElementFactory(typeof(TextBlock));

            propertyTypeDataTemplate.VisualTree = propertyTypeTextBlockFrameworkElementFactory;

            Binding propertyTypeBinding = new Binding("PropertyType");

            propertyTypeBinding.Converter = new TypeToTypeNameValueConverter();

            propertyTypeTextBlockFrameworkElementFactory.SetBinding(TextBlock.TextProperty, propertyTypeBinding);

            #endregion
            #region Default 그리드 뷰 컬럼

            GridViewColumn defaultGridViewColumn = new GridViewColumn();

            defaultGridViewColumn.Header               = "Default";
            defaultGridViewColumn.Width                = double.NaN;
            defaultGridViewColumn.DisplayMemberBinding = new Binding("DefaultMetadata.DefaultValue");

            gridView.Columns.Add(defaultGridViewColumn);

            #endregion
            #region ReadOnly 그리드 뷰 컬럼

            GridViewColumn readOnlyGridViewColumn = new GridViewColumn();

            readOnlyGridViewColumn.Header               = "Read-Only";
            readOnlyGridViewColumn.Width                = double.NaN;
            readOnlyGridViewColumn.DisplayMemberBinding = new Binding("ReadOnly");

            gridView.Columns.Add(readOnlyGridViewColumn);

            #endregion
            #region Flags 그리드 뷰 컬럼

            GridViewColumn flagsGridViewColumn = new GridViewColumn();

            flagsGridViewColumn.Header = "Flags";
            flagsGridViewColumn.Width  = double.NaN;

            gridView.Columns.Add(flagsGridViewColumn);

            DataTemplate flagsDataTemplate = new DataTemplate();

            flagsGridViewColumn.CellTemplate = flagsDataTemplate;

            FrameworkElementFactory flagsTextBlockFrameworkElementFactory = new FrameworkElementFactory(typeof(TextBlock));

            flagsDataTemplate.VisualTree = flagsTextBlockFrameworkElementFactory;

            Binding flagsBinding = new Binding("DefaultMetadata");

            flagsBinding.Converter = new FPMToFPMOValueConverter();

            flagsTextBlockFrameworkElementFactory.SetBinding(TextBlock.TextProperty, flagsBinding);

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 타입 속성 변경시 콜백 처리하기 - TypePropertyChangedCallback(d, e)

        /// <summary>
        /// 타입 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void TypePropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            DependencyPropertyListView listView = d as DependencyPropertyListView;

            Type type = e.NewValue as Type;

            listView.ItemsSource = null;

            if(type != null)
            {
                SortedList<string, DependencyProperty> list = new SortedList<string,DependencyProperty>();

                FieldInfo[] fieldInfoArray = type.GetFields();

                foreach(FieldInfo fieldInfo in fieldInfoArray)
                {
                    if(fieldInfo.FieldType == typeof(DependencyProperty))
                    {
                        list.Add(fieldInfo.Name, (DependencyProperty)fieldInfo.GetValue(null));
                    }
                }

                listView.ItemsSource = list.Values;

                GridView gridView = listView.View as GridView;

                if(gridView != null)
                {
                    foreach(GridViewColumn column in gridView.Columns)
                    {
                        column.Width = 0;
                        column.Width = double.NaN;
                    }
                }
            }
        }

        #endregion
    }
}