﻿using System;
using System.Globalization;
using System.Windows;
using System.Windows.Data;

namespace TestProject
{
    /// <summary>
    /// FrameworkPropertyMetadata ↔ FrameworkPropertyMetadataOptions 값 변환자
    /// </summary>
    public class FPMToFPMOValueConverter : IValueConverter
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 변환하기 - Convert(value, type, parameter, cultureInfo)

        /// <summary>
        /// 변환하기
        /// </summary>
        /// <param name="value">값</param>
        /// <param name="type">타입</param>
        /// <param name="parameter">매개 변수</param>
        /// <param name="cultureInfo">CultureInfo 객체</param>
        /// <returns>변환 값</returns>
        public object Convert(object value, Type type, object parameter, CultureInfo cultureInfo)
        {
            FrameworkPropertyMetadataOptions frameworkPropertyMetadataOptions = 0;

            FrameworkPropertyMetadata frameworkPropertyMetadata = value as FrameworkPropertyMetadata;

            if(frameworkPropertyMetadata == null)
            {
                return null;
            }

            if(frameworkPropertyMetadata.AffectsMeasure)
            {
                frameworkPropertyMetadataOptions |= FrameworkPropertyMetadataOptions.AffectsMeasure;
            }

            if(frameworkPropertyMetadata.AffectsArrange)
            {
                frameworkPropertyMetadataOptions |= FrameworkPropertyMetadataOptions.AffectsArrange;
            }

            if(frameworkPropertyMetadata.AffectsParentMeasure)
            {
                frameworkPropertyMetadataOptions |= FrameworkPropertyMetadataOptions.AffectsParentMeasure;
            }

            if(frameworkPropertyMetadata.AffectsParentArrange)
            {
                frameworkPropertyMetadataOptions |= FrameworkPropertyMetadataOptions.AffectsParentArrange;
            }

            if(frameworkPropertyMetadata.AffectsRender)
            {
                frameworkPropertyMetadataOptions |= FrameworkPropertyMetadataOptions.AffectsRender;
            }

            if(frameworkPropertyMetadata.Inherits)
            {
                frameworkPropertyMetadataOptions |= FrameworkPropertyMetadataOptions.Inherits;
            }

            if(frameworkPropertyMetadata.OverridesInheritanceBehavior)
            {
                frameworkPropertyMetadataOptions |= FrameworkPropertyMetadataOptions.OverridesInheritanceBehavior;
            }

            if(frameworkPropertyMetadata.IsNotDataBindable)
            {
                frameworkPropertyMetadataOptions |= FrameworkPropertyMetadataOptions.NotDataBindable;
            }

            if(frameworkPropertyMetadata.BindsTwoWayByDefault)
            {
                frameworkPropertyMetadataOptions |= FrameworkPropertyMetadataOptions.BindsTwoWayByDefault;
            }

            if(frameworkPropertyMetadata.Journal)
            {
                frameworkPropertyMetadataOptions |= FrameworkPropertyMetadataOptions.Journal;
            }

            return frameworkPropertyMetadataOptions;
        }

        #endregion
        #region 역변환하기 - ConvertBack(value, type, parameter, cultureInfo)

        /// <summary>
        /// 역변환하기
        /// </summary>
        /// <param name="value">값</param>
        /// <param name="type">타입</param>
        /// <param name="parameter">매개 변수</param>
        /// <param name="cultureInfo">CultureInfo 객체</param>
        /// <returns>역변환 값</returns>
        public object ConvertBack(object value, Type type, object parameter, CultureInfo cultureInfo)
        {
            return new FrameworkPropertyMetadata(null, (FrameworkPropertyMetadataOptions)value);
        }

        #endregion
    }
}