﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ExploreDependencyProperties()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            Width      = 800;
            Height     = 600;
            Title      = "의존 속성 탐색하기";
            FontFamily = new FontFamily("나눔고딕코딩");
            FontSize   = 16;

            Grid grid = new Grid();

            Content = grid;

            #region 왼쪽 컬럼 정의를 생성한다.

            ColumnDefinition leftColumnDefinition = new ColumnDefinition();

            leftColumnDefinition.Width = new GridLength(1, GridUnitType.Star);

            grid.ColumnDefinitions.Add(new ColumnDefinition());

            #endregion
            #region 가운데 컬럼 정의를 생성한다.

            ColumnDefinition centerColumnDefinition = new ColumnDefinition();

            centerColumnDefinition.Width = GridLength.Auto;

            grid.ColumnDefinitions.Add(centerColumnDefinition);

            #endregion
            #region 오른쪽 컬럼 정의를 생성한다.

            ColumnDefinition rightColumnDefinition = new ColumnDefinition();

            rightColumnDefinition.Width = new GridLength(3, GridUnitType.Star);

            grid.ColumnDefinitions.Add(rightColumnDefinition);

            #endregion

            #region 클래스 계층 트리 뷰를 생성한다.

            ClassHierarchyTreeView classHierarchyTreeView = new ClassHierarchyTreeView(typeof(DependencyObject));

            grid.Children.Add(classHierarchyTreeView);

            Grid.SetColumn(classHierarchyTreeView, 0);

            #endregion
            #region 그리드 스플리터를 생성한다.

            GridSplitter gridSplitter = new GridSplitter();

            gridSplitter.HorizontalAlignment = HorizontalAlignment.Center;
            gridSplitter.VerticalAlignment   = VerticalAlignment.Stretch;
            gridSplitter.Width               = 6;

            grid.Children.Add(gridSplitter);

            Grid.SetColumn(gridSplitter, 1);

            #endregion
            #region 의존 속성 리스트 뷰를 생성한다.

            DependencyPropertyListView dependencyPropertyListView = new DependencyPropertyListView();

            grid.Children.Add(dependencyPropertyListView);

            Grid.SetColumn(dependencyPropertyListView, 2);

            #endregion

            dependencyPropertyListView.SetBinding(DependencyPropertyListView.TypeProperty, "SelectedItem.Type");

            dependencyPropertyListView.DataContext = classHierarchyTreeView;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        [STAThread]
        public static void Main()
        {
            Application application = new Application();

            application.Run(new MainWindow());
        }

        #endregion
    }
}